# ⚛️ 💼 React Form

React hooks for managing form state and lifecycle

<a href="https://travis-ci.org/tannerlinsley/react-form" target="\_parent">
  <img alt="" src="https://travis-ci.org/tannerlinsley/react-form.svg?branch=master" />
</a>
<a href="https://npmjs.com/package/react-form" target="\_parent">
  <img alt="" src="https://img.shields.io/npm/dm/react-form.svg" />
</a>
<a href="https://spectrum.chat/react-form">
  <img alt="Join the community on Spectrum" src="https://withspectrum.github.io/badge/badge.svg" />
</a>
<a href="https://github.com/tannerlinsley/react-form" target="\_parent">
  <img alt="" src="https://img.shields.io/github/stars/tannerlinsley/react-form.svg?style=social&label=Star" />
</a>
<a href="https://twitter.com/tannerlinsley" target="\_parent">
  <img alt="" src="https://img.shields.io/twitter/follow/tannerlinsley.svg?style=social&label=Follow" />
</a>

<!--
## Features
- Put some features here
-->

## Intro

React Form is currently in **alpha**! This means:

- The existing API is not stable. Expect major changes/additions **as minor and patch releases** while use-cases evolve.
- It is not recommended for **mission critical** production code.

## Table of Contents

- [Installation](#installation)
- [Quick Example](#quick-example)

## Installation

```bash
$ yarn add react-form
# or
$ npm i react-form --save
```

## Quick Start

This will render a very basic form:

```js
import React from 'react'
import { useForm, useField } from 'react-form'

// Build an input field
const InputField = React.forwardRef((
  { field, validate, ...rest },
  ref
) => {
  // Use the useField hook with form props to access form state
  const {
    field: { value, meta: { error, isTouched }, setValue, setMeta }
  } = useField(field, {
    validate,
  })

  // Build your form input
  return (
    <>
      <input
        ref={ref}
        // Use setValue to update form values
        onChange={e => setValue(e.target.value)}
        // Use setMeta to update form touch state
        onBlur={() => setMeta({isTouched: true})}
        value={value}
        {...rest}
      />
      {
        // Use isTouched and error state to show errors
        isTouched && error ? <em>{error}</em> : null
      }
    </>
  )
})

function MyForm() {
  // Pass default values (be sure to memoize any options used)
  const defaultValues = React.useMemo(
    () => ({
      foo: 'hello',
      bar: {
        baz: 'world'
      }
    }),
    []
  )

  // Use the useForm hook to create a form instance
  const { Form } = useForm({
    // Pass the default values
    defaultValues,
    // Handle form submission
    onSubmit: values => {
      console.log('These are the values:', values)
    }
  })

  return (
    // Use the built-in Form component provided by the hook
    // (or optionally construct your own with the `handleSubmit` function)
    <Form>
      <InputField field="foo" />
      <InputField field="bar.baz" validate={value => !value ? 'Baz is required!' : false} />
      <button type="submit">Submit</button>
    </Form>
  )
}
```

## Documentation

Complete documentation is **coming soon**.

Any sparse documentation available in this Readme is being progressively improved as the API evolves.

## API

React Form exposes these top-level exports:

- `useForm` - The primary hook for creating a form.
- `useField` - A hook for utilizing form state and lifecycle on the field level.
